﻿using UnityEngine;
using System.Collections;

namespace RatKing {

	[System.Serializable]
	public class Anim {
		public string name;
		[System.Serializable]
		public class Frames {
			public Texture2D[] frames;
			public float side = 1f;
		}
		public Frames[] textures; // 0 == front, 1 == side, 2 == back
		public float speed = 1f;
	}

	public class Billboard : MonoBehaviour {
		public Transform fromUp;
		public Camera cam;
		public Vector3 eulerAdd = Vector3.zero;
		public Anim.Frames[] textures;
		public float animSpeed = 1f;
		//
		[System.NonSerialized]
		public Vector3 velocity = Vector3.zero;
		Material mat;
		int curTexIndex;
		float animTime;
		string curAnimName;
		float animFactor;
		Anim nextAnim;
		float animChangeTimer = 0f;
		float mirrorChangeTimer = 0f;
		//
		int dir = 0; // front, side, back
		float side = 1f; // mirrored or not

		//

		void Awake() {
			if (cam == null) {
				cam = Camera.main;
			}
			mat = GetComponent<Renderer>().material;
		}

		void Update() {
			if (velocity.sqrMagnitude > 0.0000001f) {
				var cross = Vector3.Cross(cam.transform.up, velocity);
				if (Mathf.Abs(cross.z) > 0.1f) {
					side = (cross.z < 0f ? -1f : 1f);
					side *= (textures != null && textures.Length > 0) ? textures[dir % textures.Length].side : 1f;
				}
				if (Mathf.Abs(cross.x) > 0.1f || Mathf.Abs(cross.z) > 0.1f) {
					dir = 1;
					if (Mathf.Abs(cross.x) > Mathf.Abs(cross.z)) {
						dir = cross.x < 0f ? 0 : 2; // front | back
					}
				}
			}
			// animation
			if (nextAnim != null && nextAnim.name != curAnimName && animChangeTimer <= Time.time) {
				ChangeToNextAnim();
			}
			if (textures != null & textures.Length > 0) {
				if (animTime < Time.time) {
					animTime = Time.time + 1f / (animFactor * animSpeed);
					var tex = textures[dir % textures.Length];
					curTexIndex = (curTexIndex + 1) % tex.frames.Length;
					mat.mainTexture = tex.frames[curTexIndex % tex.frames.Length];
				}
			}
			// rotation
			transform.rotation = Quaternion.LookRotation(cam.transform.position - transform.position, fromUp != null ? fromUp.up : Vector3.up) * Quaternion.Euler(eulerAdd);
			Vector3 ls = transform.localScale;
			if (mirrorChangeTimer < Time.time) {
				ls.x = Mathf.Abs(ls.x) * side;
				mirrorChangeTimer = Time.time + 0.2f; // only change mirror every 0.2 secs max
			}
			transform.localScale = ls;
		}

		//

		public void SetAnim(Anim anim, float factor = 1f) {
			animFactor = Mathf.Max(0.0001f, factor);
			if (nextAnim == null || nextAnim != anim) {
				nextAnim = anim;
			}
			if (curAnimName == anim.name) {
				return;
			}
			if (animChangeTimer > Time.time) {
				return;
			}
			animChangeTimer = Time.time + 0.2f; // only change anim every 0.2 secs max
			ChangeToNextAnim();
		}

		void ChangeToNextAnim() {
			curAnimName = nextAnim.name;
			textures = nextAnim.textures;
			animSpeed = nextAnim.speed > 0f ? nextAnim.speed : 0.0001f;
			animTime = Time.time + 1f / animSpeed;
			var tex = textures[dir % textures.Length];
			curTexIndex = Random.Range(0, tex.frames.Length);
			mat.mainTexture = tex.frames[curTexIndex];
			nextAnim = null;
		}
	}

}