﻿using UnityEngine;
using System.Collections;

namespace RatKing {
	
public class Box2Creature : MonoBehaviour {
	public float legLength = 0.75f;
	public Collider foot;
	public float walkForce = 30.0f;
	public float walkFriction = 0.9f;
	public float airFriction = 0.99f;
	public float jumpUpForce = 150.0f;
	public float jumpForwardForce = 50.0f;
	public float standForce = 0.25f;
	public float rotateForce = 0.5f;
	public float torqueAddFactor = 0.25f;
	public float walkTime = 0.5f;
	public GameObject particlesDust;
	//
	// Vector3 targetForward = Vector3.forward;
	public Vector3 targetForward { get; private set; }
	public Vector3 forward { get; private set; }
	//
	public bool walking { get; private set; }
	public int onFloor { get; private set; }
	public int jumped { get; private set; }
	public Rigidbody rb { get; private set; }
	//
	static int layerFloorMask;
	float walkTimer;
	ParticleSystem.EmissionModule dustEmis;

	//

	void Awake() {
		forward = Vector3.forward;
		targetForward = Vector3.forward;
	}

	void Start() {
		rb = GetComponent<Rigidbody>();
		layerFloorMask = ~(1 << LayerMask.NameToLayer("Player")); // TODO
	}

	//

	void OnCollisionStay(Collision coll) {
		// print(coll.contacts.Length + " " + coll.contacts[0].thisCollider);
		if (coll.contacts.Length > 0 && coll.contacts[0].thisCollider == foot)
			onFloor++;
	}

	void FixedUpdate() {
		float angle;

		forward = Vector3.Slerp(forward, targetForward, 0.5f);

		angle = Vector3.Angle(transform.forward, forward);
		if (angle > 0f) {
			Vector3 right = Vector3.Cross(transform.forward, forward).normalized;
			rb.AddTorque(right * angle * rotateForce);
			Debug.DrawRay(transform.position, right * 1.5f, Color.red);
		}

		angle = Vector3.Angle(transform.up, Vector3.up);
		if (angle > 0f) {
			Vector3 right = Vector3.Cross(transform.up, Vector3.up).normalized;
			rb.AddTorque(right * angle * standForce);
			Debug.DrawRay(transform.position, right, Color.blue);
		}

		float f = (angle / 180.0f);
		rb.angularVelocity *= Mathf.Sqrt(Mathf.Sqrt(f)) + torqueAddFactor;

		// print(jumped + " " + Physics.Raycast(transform.position, -transform.up, legLength, layerFloorMask));
		onFloor = (jumped == 0 && Physics.Raycast(transform.position, -transform.up, legLength, layerFloorMask)) ? (onFloor + 1) : 0;
		//onFloor += (jumped == 0 && Physics.Raycast(transform.position, -transform.up, legLength, layerFloorMask)) ? 1 : 0;

		Vector3 v = rb.velocity;
		if (onFloor > 0) {
			v.x *= walkFriction;
			v.z *= walkFriction;
		}
		else {
			v.x *= airFriction;
			v.z *= airFriction;
		}
		rb.velocity = v;

		if (walking) {
			rb.AddForce(forward * walkForce, ForceMode.Force);
			if (onFloor > 0 && rb.velocity.magnitude > 0.1f) {
				if (walkTimer < Time.time) {
					if (particlesDust != null) {
						var p = transform.position;
						p.y = 0.5f;
						Instantiate(particlesDust, p, transform.rotation);
					}
					Main.inst.sndWalk.Play(transform);
					walkTimer = Time.time + walkTime;
				}
			}
		}

#if UNITY_EDITOR
		Debug.DrawRay(transform.position, forward * 2.0f, Color.white);
		Debug.DrawRay(transform.position, transform.up * -legLength, Color.yellow);
#endif

		jumped--;
		jumped = Mathf.Clamp(jumped, 0, 9);
		onFloor--;
		onFloor = Mathf.Clamp(onFloor, 0, 4);
	}

	//

	public void SetMoveAxes(float v, float h) {
		Vector3 f = Vector3.forward * v + Vector3.right * h;
		if (f.sqrMagnitude > 0.001f) {
			targetForward = f.sqrMagnitude > 1f ? f.normalized : f;
		}
		walking = (v != 0f || h != 0f) && onFloor > 0;
	}

	public void Jump() {
		if (onFloor > 0 && jumped == 0) {
			float f = Vector3.Dot(transform.forward, rb.velocity.normalized);
			rb.AddForce(transform.up * jumpUpForce + transform.forward * f * jumpForwardForce, ForceMode.Force);
			jumped = 8;
		}
	}

	//

	void OnValidate() {
		walkFriction = Mathf.Clamp01(walkFriction);
	}

#if UNITY_EDITOR
	void OnDrawGizmos() {
		Gizmos.color = onFloor > 0 ? Color.green : Color.red;
		Gizmos.DrawCube(transform.position + transform.up, Vector3.one * 0.4f);
		if (!Application.isPlaying) {
			Gizmos.color = Color.yellow;
			Gizmos.DrawRay(transform.position, transform.up * -legLength);
		}
	}
#endif
}
	
}