﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace RatKing {

	public class Cauldron : MonoBehaviour {
		public Player myPlayer;
		public GameObject trigObject;
		public Transform putDummy;
		//
		Stack<int> ingredients = new Stack<int>(2);
		List<int> taskCompleted = new List<int>();

		//

		// helper class trigger
		public class CauldronTrigger : MonoBehaviour {
			public Cauldron myCauldron;
			//
			List<Player> playersNear = new List<Player>();
			
			//

			void Awake() {
				GetComponent<Collider>().isTrigger = true;
			}

			void Update() {
				bool showTip = false;
				for (var iter = playersNear.GetEnumerator(); iter.MoveNext(); ) {
					if (iter.Current.CanDropInCauldron(myCauldron)) {
						showTip = true;
						break;
					}
				}
				if (showTip) {
					Main.inst.ShowTip(myCauldron.transform, "Put!", 1.75f);
				}
				else {
					Main.inst.UnshowTip(myCauldron.transform);
				}
			}

			void OnTriggerStay(Collider coll) {
				// if the player is near and can do something with the cauldron, show a message
				var p = coll.attachedRigidbody.GetComponent<Player>();
				if (p != null && !playersNear.Contains(p)) {
					playersNear.Add(p);
					p.SetCauldron(myCauldron);
				}
			}

			void OnTriggerExit(Collider coll) {
				var p = coll.attachedRigidbody.GetComponent<Player>();
				if (p != null && playersNear.Contains(p)) {
					p.UnsetCauldron(myCauldron);
					playersNear.Remove(p);
				}
			}
		}

		//

		void Awake() {
			var trigger = trigObject.AddComponent<CauldronTrigger>();
			trigger.myCauldron = this;
		}

		//

		public int TaskCompleted() {
			return taskCompleted.Count;
		}

		public void AddIngredient(int prefabIndex, Player byPlayer) {
			if (Main.inst.reagentPrefabs[prefabIndex].type == Collectible.Type.Bomb) {
				Main.inst.sndBomb.Play(transform.position);
				PopLastIngredient();
				Instantiate(Main.inst.effectPrefabCauldronBomb, putDummy.position, Quaternion.identity);
				Main.inst.PlayerDidBad(byPlayer, myPlayer != byPlayer ? 5 : 3);
				byPlayer.AddScore(myPlayer != byPlayer ? Main.inst.pointsPlantBombAtOther : Main.inst.pointsPlantBombAtSelf);
			}
			else {
				// check: correct ingredient?
				if (Main.inst.curTask.FindAll(v => v == prefabIndex).Count > taskCompleted.FindAll(v => v == prefabIndex).Count) {
					Main.inst.sndIngredientRight.Play(transform.position);
					taskCompleted.Add(prefabIndex);
					byPlayer.AddScore(byPlayer == myPlayer ? Main.inst.pointsAddCorrectAtSelf : Main.inst.pointsAddCorrectAtOther);
					Instantiate(Main.inst.effectPrefabCauldronGood, putDummy.position, Quaternion.identity);
				}
				else {
					Instantiate(Main.inst.effectPrefabCauldronBad, putDummy.position, Quaternion.identity);
					Main.inst.sndIngredientFail.Play(transform.position);
					if (byPlayer == myPlayer) {
						byPlayer.AddScore(Main.inst.pointsAddWrongAtSelf);
					}
					else {
						myPlayer.AddScore(Main.inst.pointsAddWrongByOther);
					}
					if (byPlayer != myPlayer) {
						Main.inst.PlayerDidBad(byPlayer, 1);
					}
				}
				ingredients.Push(prefabIndex);
				if (taskCompleted.Count == Main.inst.taskCount) {
					// print("END"); // TODO
					Main.inst.End();
				}
			}
		}

		public void PopLastIngredient() {
			if (ingredients.Count == 0) {
				return;
			}
			int index = ingredients.Pop();
			Main.inst.SpawnIngredient(index, putDummy, PopMode.Cauldron);
			if (taskCompleted.Contains(index)) {
				taskCompleted.Remove(index);
				myPlayer.AddScore(-Main.inst.pointsAddCorrectAtSelf);
			}
		}

		//

#if UNITY_EDITOR
		void OnDrawGizmos() {
			var sphere = trigObject.GetComponent<SphereCollider>();
			if (sphere != null) {
				Gizmos.DrawWireSphere(sphere.transform.position, sphere.radius);
			}
		}
#endif
	}

}