﻿using UnityEngine;
using System.Collections;

namespace RatKing {

	public class Collectible : MonoBehaviour {
		public enum Type {
			Ingredient,
			Bomb
		}
		public Type type;
		public string desc;
		public float size = 1f;
		public float height = 0.5f;
		//
		bool collected;
		bool putting;
		float timeAdd;
		//
		Rigidbody rb;
		[System.NonSerialized]
		public int prefabIndex = -1;
		[System.NonSerialized]
		public int spawnerIndex = -1;
		public Billboard billboard;

		//

		void Awake() {
			rb = GetComponent<Rigidbody>();
			billboard = gameObject.GetComponentInChildren<Billboard>();
			billboard.transform.localScale = Vector3.one * size;
			timeAdd = Random.Range(0f, Mathf.PI * 2f);
		}

		void Update() {
			var ea = billboard.eulerAdd;
			ea.z = Mathf.Sin((Time.time) * 2f + timeAdd) * 10f;
			billboard.eulerAdd = ea;
		}

		/*void OnTriggerEnter(Collider coll) {
			if (collected) {
				return;
			}
			var p = coll.attachedRigidbody.GetComponent<Player>();
			if (p != null && p.CanGetCollectible(this)) {
				collected = true;
				if (rb != null) {
					rb.isKinematic = true;
				}
			}
		}*/

		//

		public bool CollectBy(Player player) {
			if (!putting && !collected && player != null && player.CanGetCollectible(this)) {
				collected = true;
				Main.inst.Despawn(this);
				if (rb != null) {
					rb.isKinematic = true;
				}
				Main.inst.sndIngredientTake.Play(transform.position);
				return true;
			}
			return false;
		}

		public bool PutIntoCauldron(Cauldron cauldron, Player byPlayer) {
			if (putting || cauldron == null) {
				return false;
			}
			putting = true;
			StartCoroutine(PutIntoCauldronCR(cauldron, byPlayer));
			return true;
		}
		IEnumerator PutIntoCauldronCR(Cauldron cauldron, Player byPlayer) {
			Vector3 startPos = transform.position;
			Vector3 endPos = cauldron.putDummy.position;
			float factor = 1f;
			Vector3 startScale = transform.localScale;
			while (factor > 0f) {
				factor -= Time.deltaTime;
				var pos = Vector3.Lerp(startPos, endPos, 1f - factor);
				pos.y += Mathf.Sin(Mathf.PI * factor) * 2f;
				transform.position = pos;
				transform.localScale = startScale * (0.5f + factor * 0.5f);
				yield return null;
			}
			cauldron.AddIngredient(prefabIndex, byPlayer);
			Main.inst.RegisterFreeSpawner(spawnerIndex);
			Destroy(gameObject);
		}

		public void PutOutOfCauldron(Vector3 pos) {
			if (putting) {
				return;
			}
			putting = true;
			StartCoroutine(PutOutOfCauldronCR(pos));
			return;
		}
		IEnumerator PutOutOfCauldronCR(Vector3 targetPos) {
			Vector3 startPos = transform.position;
			Vector3 endPos = targetPos;
			float factor = 1f;
			Vector3 startScale = transform.localScale;
			while (factor > 0f) {
				factor -= Time.deltaTime;
				var pos = Vector3.Lerp(startPos, endPos, 1f - factor);
				pos.y += Mathf.Sin(Mathf.PI * factor) * 2f;
				transform.position = pos;
				transform.localScale = startScale * (0.5f + (1f - factor) * 0.5f);
				yield return null;
			}
			putting = false;
		}

		public void PutOutOfSnape() {
			if (putting) {
				return;
			}
			putting = true;
			StartCoroutine(PutOutOfSnapeCR());
		}
		IEnumerator PutOutOfSnapeCR() {
			Vector3 startPos = Main.inst.teacher.transform.position + Vector3.up * 2f;
			Vector3 endPos = transform.position;
			float factor = 1f;
			while (factor > 0f) {
				factor -= Time.deltaTime;
				var pos = Vector3.Lerp(startPos, endPos, 1f - factor);
				pos.y += Mathf.Sin(Mathf.PI * factor) * 2f;
				transform.position = pos;
				yield return null;
			}
			putting = false;
		}

		public void Drop() {
			if (putting) {
				return;
			}
			RaycastHit hit;
			if (Physics.Raycast(transform.position + Vector3.up * 50f, Vector3.down, out hit, 100f, (1 << LayerMask.NameToLayer("Floor")))) {
				putting = true;
				StartCoroutine(DropCR(hit.point + Vector3.up * height));
			}
		}
		IEnumerator DropCR(Vector3 endPos) {
			Vector3 startPos = transform.position;
			float factor = 1f;
			while (factor > 0f) {
				factor -= Time.deltaTime * 2f;
				var pos = Vector3.Lerp(startPos, endPos, 1f - factor);
				// pos.y += Mathf.Sin(Mathf.PI * factor) * 2f;
				transform.position = pos;
				yield return null;
			}
			Main.inst.ReturnToCollectibles(this);
			collected = false;
			putting = false;
		}
	}

}