﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using DG.Tweening;
using TMPro;

namespace RatKing {

	public class Tip {
		public Transform messenger;
		public string message;
		public Transform display;
		public float height;
	}

	public enum PopMode {
		None,
		Snape,
		Cauldron
	}

	public class Main : MonoBehaviour {
		public static Main inst { get; private set; }
		public GameObject tipPrefab;
		public Transform spawnReagents;
		public Collectible[] reagentPrefabs;
		public GameObject[] reagentQuads;
		public float reagentMinDist = 2f;
		public GameObject effectPrefabCauldronBomb;
		public GameObject effectPrefabCauldronGood;
		public GameObject effectPrefabCauldronBad;
		public TextMeshPro blackboardText;
		public Teacher teacher;
		public Player[] players;
		public int taskCount = 3;
		public float blubberTimeMin = 5f;
		public float blubberTimeMax = 10f;
		[Header("Sounds")]
		public AudioClip music;
		public Base.Sound sndBlubber;
		public Base.Sound sndBomb;
		public Base.Sound sndHitFail;
		public Base.Sound sndHitOuch;
		public Base.Sound sndIngredientFail;
		public Base.Sound sndIngredientRight;
		public Base.Sound sndIngredientTake;
		public Base.Sound sndPotionMade;
		public Base.Sound sndPotionWrite;
		public Base.Sound sndWalk;
		[Header("UI")]
		public TextMeshPro title;
		public GameObject[] screenWin;
		public Renderer background;
		public Color[] winColors;
		public Transform[] blackboardDummies;
		public GameObject escapeText;
		[Header("Points")]
		public int pointsPlantBombAtOther = 3;
		public int pointsPlantBombAtSelf = 0;
		public int pointsAddCorrectAtSelf = 10;
		public int pointsAddCorrectAtOther = -5;
		public int pointsAddWrongAtSelf = -2;
		public int pointsAddWrongByOther = -1;
		public int pointsRewardCompletePotion = 10;
		//
		Dictionary<Transform, Tip> tips = new Dictionary<Transform,Tip>(3);
		List<Collectible> collectibles = new List<Collectible>(3);
		Dictionary<Transform, Collectible> collectiblesTipped = new Dictionary<Transform, Collectible>(2);
		//
		public bool ended { get; private set; }
		public List<int> curTask { get; private set; }
		List<Transform> allSpawners = new List<Transform>();
		List<int> freeSpawners = new List<int>();
		float blubberTimer;

		//

#if UNITY_EDITOR
		void OnGUI() {
			int i = 0;
			for (var iter = collectiblesTipped.GetEnumerator(); iter.MoveNext(); ++i) {
				GUI.Label(new Rect(10, 10 + i * 20, 300, 20), iter.Current.Key.name + " => " + iter.Current.Value.name);
			}
		}
#endif

		void Awake() {
			blubberTimer = Time.time + blubberTimeMin;
			if (inst != null) {
				DestroyImmediate(inst.gameObject);
			}
			inst = this;
			foreach (Transform t in spawnReagents) {
				allSpawners.Add(t);
			}
			background.gameObject.SetActive(true);
			title.gameObject.SetActive(true);
		}

		void Start() {
			blackboardText.text = new[] { "Snape sux", "Snape is stupid", "Snape go home", "Who the f*ck is Snape" }[Random.Range(0, 4)];
			teacher.GoToBlackboard();
			foreach (var b in blackboardDummies) {
				for (int i = b.childCount - 1; i >= 0; --i) {
					Destroy(b.GetChild(i).gameObject);
				}
			}
			Base.Music.Play(music, 0.1f);
		}

		//

		public void End() {
			Base.Music.Play(null, 0.0f, 5f);
			ended = true;
			for (int i = 0; i < players.Length; ++i) {
				if (players[i].myCauldron.TaskCompleted() == taskCount) {
					players[i].AddScore(pointsRewardCompletePotion);
				}
			}
			Player winner = players[0];
			for (int i = 1; i < players.Length; ++i) {
				if (players[i].GetScore() > winner.GetScore()) {
					winner = players[i];
				}
			}
			screenWin[winner.inputPlayerID].SetActive(true);
			screenWin[winner.inputPlayerID].transform.DORotate(new Vector3(90f, 0f, 0f), 1f).From();
			background.gameObject.SetActive(true);
			background.material.color = new Color(0f, 0f, 0f, 0f);
			DOTween.To(() => background.material.color, v => background.material.color = v, winColors[winner.inputPlayerID], 1f);
		}

		public void CreateTask() {
			curTask = new List<int>();
			var t = "Thou hast to mix ";
			for (int i = 0; i < taskCount; ++i) {
				for(;;) {
					var r = Random.Range(0, reagentPrefabs.Length);
					if (reagentPrefabs[r].type == Collectible.Type.Ingredient) {
						curTask.Add(r);
						t += " " + reagentPrefabs[r].desc + ((i < taskCount - 2) ? "," : (i < taskCount - 1) ? " and" : "!");
						var go = (GameObject)Instantiate(reagentQuads[r], blackboardDummies[i].position, blackboardDummies[i].rotation);
						go.transform.SetParent(blackboardDummies[i]);
						break;
					}
				}
			}

			for (int i = 0; i < allSpawners.Count; ++i) {
				var clb = SpawnIngredient(Random.Range(0, reagentPrefabs.Length), allSpawners[i], PopMode.Snape);
				clb.spawnerIndex = i;
			}
			
			blackboardText.text = t;
		}

		public Collectible SpawnIngredient(int prefabIndex, Transform spawn, PopMode popMode = PopMode.None) {
			var go = (GameObject)Instantiate(reagentPrefabs[prefabIndex].gameObject, spawn.position, spawn.rotation);
			var collectible = go.GetComponent<Collectible>();
			collectible.prefabIndex = prefabIndex;
			collectibles.Add(collectible);
			if (popMode == PopMode.Cauldron) {
				for(;;) {
					var circle = 3f * Random.insideUnitCircle.normalized;
					var pos = spawn.position + new Vector3(circle.x, 0f, circle.y);
					RaycastHit hit;
					if (Physics.Raycast(pos + Vector3.up * 50f, Vector3.down, out hit, 100f, (1 << LayerMask.NameToLayer("Floor")))) {
						collectible.PutOutOfCauldron(hit.point + Vector3.up * collectible.height);
						return collectible;
					}
				} 
			}
			else if (popMode == PopMode.Snape) {
				collectible.PutOutOfSnape();
			}
			return collectible;
		}

		public void Despawn(Collectible coll) {
			collectibles.Remove(coll);
			UnshowTip(coll.transform);
			// TODO remove from collectiblesTipped
		}

		public void RegisterFreeSpawner(int index) {
			if (index < 0 || freeSpawners.Contains(index)) {
				return;
			}
			freeSpawners.Add(index);
		}

		public void ReturnToCollectibles(Collectible coll) {
			if (coll != null && !collectibles.Contains(coll)) {
				collectibles.Add(coll);
			}
		}

		//

		float escapeTimer = 0f;

		void Update() {
			if (Input.GetKeyDown(KeyCode.Escape)) {
				if (escapeTimer > Time.time) {
					Application.Quit();
				}
				escapeTimer = Time.time + 2f;
			}
			escapeText.SetActive(escapeTimer > Time.time);

			if (blubberTimer < Time.time) {
				sndBlubber.Play();
				blubberTimer = Time.time + Random.Range(blubberTimeMin, blubberTimeMax);
			}

			if (title.gameObject.name != "Title Outfade") {
				for (int i = players.Length - 1; i >= 0; --i) {
					if (players[i].WasMoved()) {
						title.gameObject.name = "Title Outfade";
						var c = title.color;
						c.a = 0f;
						DOTween.To(() => title.color, v => title.color = v, c, 2f).OnComplete(() => title.gameObject.SetActive(false));
						var d = background.material.color;
						d.a = 0f;
						DOTween.To(() => background.material.color, v => background.material.color = v, c, 2f).OnComplete(() => background.gameObject.SetActive(false));
						break;
					}
				}
			}

			if (Input.GetKeyDown(KeyCode.F12)) {
				Base.Helpers.Debug.CreateScreenshot("PotParty");
			}
			for (var iter = tips.GetEnumerator(); iter.MoveNext(); ) {
				var tip = iter.Current.Value;
				tip.display.position = tip.messenger.position + Vector3.up * tip.height;
			}

			if (Main.inst.ended) {
				return;
			}
			if (teacher.IsLooking() && freeSpawners.Count > 0) {
				int spawnIndex = freeSpawners[0];
				freeSpawners.RemoveAt(0);
				var clt = SpawnIngredient(Random.Range(0, reagentPrefabs.Length), allSpawners[spawnIndex], PopMode.Snape);
				clt.spawnerIndex = spawnIndex;
			}
		}

		public Collectible GetNearestReagent(Transform trans, bool canCollect = true) {
			var dist = reagentMinDist * reagentMinDist;
			Collectible nearest = null;
			for (var iter = collectibles.GetEnumerator(); iter.MoveNext(); ) {
				var d = (trans.position - iter.Current.transform.position).sqrMagnitude;
				if (d < dist) {
					dist = d;
					nearest = iter.Current;
				}
			}
			if (collectiblesTipped.ContainsKey(trans) && collectiblesTipped[trans] != nearest) {
				if (collectiblesTipped[trans] != null) {
					UnshowTip(collectiblesTipped[trans].transform);
				}
				collectiblesTipped.Remove(trans);
			}
			if (nearest != null && canCollect) {
				collectiblesTipped[trans] = nearest;
				ShowTip(nearest.transform, "Take!", 1f);
			}
			return nearest;
		}

		//

		public void PlayerDidBad(Player player, int severity) {
			// severity is from 1 to 6, with 6 being a total asshole.
			if (severity >= 6 || teacher.IsLooking(player.transform)) {
				teacher.GetAngryAt(player, severity);
			}
		}

		public void Restart() {
			// UnityEngine.SceneManagement.SceneManager.UnloadScene("main");
			UnityEngine.SceneManagement.SceneManager.LoadScene("main");
		}

		//

		Transform tipParent;

		public void ShowTip(Transform messenger, string message, float height) {
			if (tips.ContainsKey(messenger)) {
				return;
			}
			if (tipParent == null) {
				tipParent = new GameObject("_TIPS_").transform;
			}
			var displayGO = (GameObject)Instantiate(tipPrefab);
			displayGO.name = tipPrefab.name + " -> " + messenger.name + " <- " + message;
			var display = displayGO.transform;
			display.SetParent(tipParent);
			display.DOScale(0.0001f, 0.3f).From().SetEase(Ease.OutBounce);
			display.GetComponentInChildren<TextMeshPro>(true).text = message;
			var tip = new Tip() { messenger = messenger, message = message, display = display, height = height };
			tips[messenger] = tip;
		}

		public void UnshowTip(Transform messenger) {
			if (tips.ContainsKey(messenger)) {
				var display = tips[messenger].display;
				var tm = display.GetComponentInChildren<TextMeshPro>();
				var c = tm.color;
				c.a = 0f;
				DOTween.To(() => tm.color, val => tm.color = val, c, 0.25f).OnComplete(() => Destroy(display.gameObject));
				tips.Remove(messenger);
			}
		}

		//

#if UNITY_EDITOR
		void OnDrawGizmos() {
			if (spawnReagents != null) {
				Gizmos.color = Color.white;
				for (int i = 0; i < spawnReagents.transform.childCount; ++i) {
					var t = spawnReagents.transform.GetChild(i);
					Gizmos.DrawSphere(t.position, 0.25f);
				}
			}
		}
#endif
	}

}