﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using TMPro;

namespace RatKing {

	public class Player : MonoBehaviour {
		public TextMeshPro scoreBoard;
		public TextMeshPro reagentCountBoard;
		public GameObject punchPrefab;
		public GameObject freezeStars;
		public Cauldron myCauldron;
		[Header("Input")]
		public int inputPlayerID = 0;
		public float punchSeconds = 0.75f;
		[Header("Animation")]
		public Billboard billboard;
		public Anim[] anims;
		//
		Box2Creature crt;
		Rewired.Player input;
		Collectible nearCollectible;
		Collectible curCollectible;
		float showScore;
		int realScore;
		Cauldron curCauldron;
		float punchTime;
		float freezeTime;
		//Color freezeCubeColor;
		//float freezeCubeColorMaxAlpha;
		float moveFactor = 1f;
		Dictionary<string, Anim> animsByName = new Dictionary<string, Anim>();
	
		//

		void Awake() {
			animsByName = new Dictionary<string, Anim>(2);
			for (int i = 0; i < anims.Length; ++i) {
				animsByName[anims[i].name] = anims[i];
			}
		}

		void Start() {
			//freezeCubeColor = freezeCube.material.color;
			//freezeCubeColorMaxAlpha = freezeCubeColor.a;
			//freezeCube.enabled = false;
			freezeStars.SetActive(false);
			crt = GetComponent<Box2Creature>();
			input = Rewired.ReInput.players.GetPlayer(inputPlayerID);
		}

		public bool WasMoved() {
			return new Vector2(crt.rb.velocity.x, crt.rb.velocity.z).magnitude > 0.2f;
		}
		
		void Update() {
			var v = input.GetAxis("Move Ver") * moveFactor;
			var h = input.GetAxis("Move Hor") * moveFactor;
			crt.SetMoveAxes(v, h);
			billboard.velocity = crt.rb.velocity;
			if (Mathf.Abs(v) > 0.1f || Mathf.Abs(h) > 0.1f) {
				billboard.SetAnim(animsByName["run"], moveFactor);
			}
			else {
				billboard.SetAnim(animsByName["idle"]);
			}
			showScore = Mathf.MoveTowards(showScore, (float)realScore, Time.deltaTime * 10f);
			var board = ""; for (int i = 0; i < myCauldron.TaskCompleted(); ++i) { board += "*"; }
			reagentCountBoard.SetText(board);
			scoreBoard.SetText(Mathf.FloorToInt(showScore).ToString());
			if (freezeTime > 0f) {
				UpdateFreeze();
			}
			nearCollectible = Main.inst.GetNearestReagent(transform, curCollectible == null);
			if (Main.inst.ended) {
				if (input.GetButtonDown("Restart")) {
					Main.inst.Restart();
				}
				return;
			}
			if (input.GetButton("Action")) {
				DoAction();
			}
			if (input.GetButton("Punch")) {
				DoPunch();
			}
		}

		void FixedUpdate() {
			if (curCollectible != null) {
				curCollectible.transform.position = Vector3.Lerp(curCollectible.transform.position, transform.position + Vector3.up * 1.5f, 0.2f);
			}
		}

		//

		void UpdateFreeze() {
			freezeTime -= Time.deltaTime;
			// float factor = 1f - Mathf.Clamp01(freezeTime / 0.1f);
			moveFactor = 1f - 0.95f * Mathf.Clamp01(freezeTime / 0.1f);
			freezeStars.transform.Rotate(0f, 0f, Time.deltaTime * 180f, Space.Self);
			if (freezeTime <= 0f) {
				// freezeCube.enabled = false;
				freezeStars.SetActive(false);
				moveFactor = 1f;
			}
			//freezeCubeColor.a = freezeCubeColorMaxAlpha * (1f - factor);
			//freezeCube.material.color = freezeCubeColor;
		}

		public void AddScore(int i) {
			realScore += i;
		}

		public int GetScore() {
			return realScore;
		}

		void DoAction() {
			if (freezeTime > 0f) {
				return;
			}
			if (curCollectible != null && curCollectible.PutIntoCauldron(curCauldron, this)) {
				curCollectible = null;
			} /*
			else if (curCollectible != null) {
				curCollectible.Drop();
				curCollectible = null;
			} */
			else if (nearCollectible != null) {
				nearCollectible.CollectBy(this);
			}
		}

		void DoPunch() {
			if (curCollectible != null) {
				curCollectible.Drop();
				curCollectible = null;
				punchTime = Time.time + punchSeconds;
				return;
			}
			if (punchTime > Time.time || freezeTime > 0f) {
				return;
			}
			punchTime = Time.time + punchSeconds;
			var rot = Quaternion.FromToRotation(Vector3.forward, crt.targetForward);
			var go = (GameObject)Instantiate(punchPrefab, transform.position, rot);
			var punch = go.GetComponent<Punch>();
			punch.Do(crt.rb, this);
		}

		public bool GetPunched(Vector3 force, Player byPlayer, float time) {
			if (freezeTime > 0f) {
				return false;
			}
			// print(name + " got hit " + force + " by " + byPlayer.name + " " + freezeTime);
			crt.rb.AddForce(force);
			Freeze(time);
			if (curCollectible != null) {
				curCollectible.Drop();
				curCollectible = null;
			}
			return true;
		}

		public void Freeze(float time) {
			freezeTime = Mathf.Max(time, freezeTime);
			// freezeCube.enabled = true;
			freezeStars.SetActive(true);
		}

		//

		public bool CanGetCollectible(Collectible collectible) {
			if (curCollectible != null) {
				return false;
			}
			curCollectible = collectible;
			return true;
		}

		public void SetCauldron(Cauldron cauldron) {
			if (curCauldron == null) {
				curCauldron = cauldron;
			}
		}

		public void UnsetCauldron(Cauldron cauldron) {
			if (curCauldron == cauldron) {
				curCauldron = null;
			}
		}

		public bool CanDropInCauldron(Cauldron cauldron) {
			return curCauldron == cauldron && curCollectible != null;
		}
	}
	
}